package com.gic.haoban.manage.api.service;

import com.gic.api.base.commons.Page;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.ExternalClerkRelatedDTO;
import com.gic.haoban.manage.api.dto.ExternalClerkRelatedShortInfoDTO;

import java.util.Date;
import java.util.List;


/**
 * Created by tgs on 2020/2/9.
 */
public interface ExternalClerkRelatedApiService {

	/**
	 * 查询导购好友关联
	 *
	 * @param wxUserId       : 微信用户id
	 * @param wxEnterpriseId : 微信企业id
	 * @param enterpriseId   : 企业id
	 * @return : 返回好友关联
	 */
	List<ExternalClerkRelatedDTO> listByWxUserIdAndWxEnterpriseId(String wxUserId, String wxEnterpriseId, String enterpriseId);

	/**
	 * 查询会员在企业下的好友关系
	 *
	 * @param memberId : 会员id
	 * @param wxEnterpriseId : 微信企业id
	 * @param enterpriseId : 企业id
	 * @return : 返回会员导购关联关系
	 */
	List<ExternalClerkRelatedDTO> listByMemberIdAndWxEnterpriseId(String memberId, String wxEnterpriseId, String enterpriseId);

	List<ExternalClerkRelatedDTO> listByEnterpriseIdAndDate(String enterpriseId,Date date);

	Page<ExternalClerkRelatedDTO> pageMemberUnionByParams(List<String> userIdList, List<String> sendMemberIds,
			String enterpriseId, BasePageInfo pageInfo);

	List<ExternalClerkRelatedDTO> listByWxEnterpriseIdAndWxUserId(String wxUserId, String wxEnterpriseId);

	List<ExternalClerkRelatedDTO> listByWxEnterpriseIdAndWxUserIds(List<String> userIdList, String wxEnterpriseId);

	/**
	 * 根据关联key查询待关联的记录-触达待完善记录
	 *
	 * @return
	 */
	ExternalClerkRelatedDTO getExternalClerkByRelationKeyAndPre(String relationKey);

	/**
	 * 根据关联key查询待关联的记录
	 *
	 * @return
	 */
	Page<ExternalClerkRelatedDTO> pageExternalClerk(String wxEnterpriseId, String wxUserId, List<Integer> statusFlags, BasePageInfo pageInfo);

	/**
	 * 推送外部联系人 更新次数以及状态
	 *
	 * @param relationKey
	 * @return
	 */
	boolean pushExternalByRelationKey(String relationKey);

	/**
	 * 推送外部联系人 更新次数以及状态
	 *
	 * @param relationKey
	 * @return
	 */
	ServiceResponse<ExternalClerkRelatedDTO> bindMemberIdExternalByRelationKey(String relationKey, String enterpriseId, String memberId, String unionid);

    /**
     * 查询clerk与union好友关系
     *
     * @param enterpriseId
     * @return
     */
    ExternalClerkRelatedDTO getClerkFriendByUnionid(String enterpriseId, String clerkId, String unionid);

    /**
     * 获取企业下的关联关系列表
     *
     * @return
     */
    List<ExternalClerkRelatedDTO> listExternalClerkByWxUserIdAncExternalUserId(String wxEnterpriseId, String wxUserId, String enterpriseId, String externalUserId);

    /**
     * 根据id查询列表
     *
     * @param wxEnterpriseId
     * @param ids
     * @return
     */
    List<ExternalClerkRelatedDTO> listByIds(String wxEnterpriseId, List<String> ids);

    /**
     * 查询导购好友关联
     *
     * @param wxEnterpriseId : 微信企业id
     * @return : 返回好友关联
     */
    Page<String> pageExternalByEnterpriseId(String wxEnterpriseId, String enterpriseId, BasePageInfo pageInfo);

    /**
     * 查询导购好友关联
     *
     * @param wxEnterpriseId : 微信企业id
     * @return : 返回好友关联
     */
    Page<String> pageExternalByEnterpriseId(String wxEnterpriseId, String enterpriseId, int pageNum, int pageSize);


	/**
     * 获取企业下的关联关系的 memberIds
     *
	 * @return
	 */
    List<String> listExternalClerkMemberIdsByMemberIds(String wxEnterpriseId, List<String> memberIds);

    /**
     * 获取企业下的关联关系列表
     *
     * @return
     */
    List<ExternalClerkRelatedShortInfoDTO> listExternalClerkByMemberIds(String wxEnterpriseId, List<String> memberIds);

    /**
     * 获取企业下的关联关系列表
     *
     * @return
     */
    List<ExternalClerkRelatedShortInfoDTO> listExternalClerkByMemberId(String memberId);

}
