package com.gic.haoban.manage.service.service.out.impl;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.gic.dubbo.entity.ProviderLocalTag;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.AlertTypeEnum;
import com.gic.haoban.manage.api.service.CheckQywxSettingApiService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RBucket;
import org.redisson.api.RMap;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * Created 2021/5/31.
 *
 * @author hua
 */
@Service
public class CheckQywxSettingApiServiceImpl implements CheckQywxSettingApiService {

    private static final Logger logger = getLogger(CheckQywxSettingApiServiceImpl.class);
    private static final String FRIEND_SETTING_KEY = "haoban-check-friend-setting";
    private static final String FRIEND_SETTING_VAL_KEY = "haoban-check-friend-setting-val";

    private static final String SELF_APP = "selfSuiteId";
    private static final String THIRD_APP = "thirdSuiteId";

    private static final String ALERT_KEY = "friend-alert:";

    private static final String ALERT_URL = "https://oapi.dingtalk.com/robot/send?access_token=435c0ddaacda849e145b64cc3a83c37f742105d88cf27b719b2dbef167436ad4";

    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Autowired
    private EnterpriseService enterpriseService;

    @Override
    public void setFriendSetting(String wxEnterpriseId, String name) {
        CheckFriendSettingDTO checkFriendSettingDTO = new CheckFriendSettingDTO();
        checkFriendSettingDTO.setName(name);
        checkFriendSettingDTO.setWxEnterpriseId(wxEnterpriseId);
        RedisUtil.setCache(FRIEND_SETTING_KEY, checkFriendSettingDTO);
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RMap<Object, Object> map = redisClient.getMap(FRIEND_SETTING_VAL_KEY);
        map.delete();
    }

    @Override
    public CheckFriendSettingDTO getFriendSetting() {
        return RedisUtil.getCache(FRIEND_SETTING_KEY, CheckFriendSettingDTO.class);
    }

    @Override
    public void saveFriendCallback(String wxEnterpriseId, ExternalUserDTO externalUserDTO, MemberUnionidRelatedDTO dto) {
        try {
            if (null == externalUserDTO) {
                messageAlert(wxEnterpriseId, dto, null);
                return;
            }
            CheckFriendSettingDTO friendSetting = this.getFriendSetting();
            if (friendSetting == null) {
                return;
            }
            if (!wxEnterpriseId.equals(friendSetting.getWxEnterpriseId())) {
                return;
            }
            if (!friendSetting.getName().equals(externalUserDTO.getName())) {
                return;
            }
            RedissonClient redisClient = RedisUtil.getRedisClient();
            RMap<Object, Object> map = redisClient.getMap(FRIEND_SETTING_VAL_KEY);
            if (SELF_APP.equals(dto.getSuiteid())) {
                map.put(SELF_APP, externalUserDTO);
            } else {
                map.put(THIRD_APP, externalUserDTO);
            }
            messageAlert(wxEnterpriseId, dto, "好友提醒：" + dto.getSuiteid());
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("通知异常：{}", e);
        }
    }

    /**
     * 告警
     *
     * @param wxEnterpriseId
     * @param dto
     */
    private void messageAlert(String wxEnterpriseId, MemberUnionidRelatedDTO dto, String titleSub) {
        titleSub = StringUtils.isBlank(titleSub) ? "加好友异常" : titleSub;
        String key = ALERT_KEY + wxEnterpriseId;
        RBucket<Object> bucket = RedisUtil.getRedisClient().getBucket(key);
        boolean b = bucket.trySet(1, 60L, TimeUnit.SECONDS);
        if (!b) {
            return;
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        ProviderLocalTag providerLocalTag = ProviderLocalTag.tag.get();
        String traceId = providerLocalTag.traceId;
        String traceUrl = "http://10.105.8.172:8780/damo-stats/mongodb/getLogMessagesView?trace_id=" + traceId;

        String title = wxEnterpriseDTO.getCorpName() + ": " + titleSub;
        Map<String, Object> map = new HashMap<>();
        Map<String, Object> link = new HashMap<>();
        map.put("msgtype", "link");
        link.put("title", title);
        link.put("text", JSONObject.toJSONString(dto));
        link.put("messageUrl", traceUrl);
        map.put("link", link);
        String post = HttpUtil.post(ALERT_URL, JSONObject.toJSONString(map));
        logger.info("alert:{}", post);
    }

    @Override
    public void alertCallback(String params) {
        logger.info("告警：{}", params);
        AlertMessageDTO messageDTO = JSONObject.parseObject(params, AlertMessageDTO.class);
        if (null == messageDTO) {
            return;
        }
        //告警是否拦截重复告警 防止批量触发
        if (StringUtils.isNotBlank(messageDTO.getKey())) {
            RBucket<Object> bucket = RedisUtil.getRedisClient().getBucket(messageDTO.getKey());
            boolean b = bucket.trySet(1, 60L, TimeUnit.SECONDS);
            if (!b) {
                return;
            }
        }

        Map<String, Object> dingMap = new HashMap<>();
        Map<String, Object> markdownMap = new HashMap<>();
        dingMap.put("msgtype", "markdown");
        dingMap.put("markdown", markdownMap);

        String text = "";
        String dingUrl = ALERT_URL;
        AlertTypeEnum anEnum = AlertTypeEnum.getEnum(messageDTO.getAlertType());
        if (anEnum != null) {
            markdownMap.put("title", anEnum.getName() + "-" + messageDTO.getAlertTitle());
            text += "标题：" + anEnum.getName() + "-" + messageDTO.getAlertTitle() + "\n\r";
            if (anEnum.getDingUrl() != null) {
                dingUrl = anEnum.getDingUrl();
            }
        } else {
            markdownMap.put("title", messageDTO.getAlertTitle());
            text += "标题：" + messageDTO.getAlertTitle() + "\n\r";
        }

        //企业信息
        if (StringUtils.isNotBlank(messageDTO.getWxEnterpriseId())) {
            WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(messageDTO.getWxEnterpriseId());
            text += "企业：" + wxEnterpriseDTO.getCorpName() + "\n\r";
        }
        // 商户信息
        if (StringUtils.isNotBlank(messageDTO.getEnterpriseId())) {
            EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(messageDTO.getEnterpriseId());
            text += "商户：" + enterpriseDTO.getEnterpriseName() + "(" + enterpriseDTO.getBrandName() + ")\n\r";
        }

        Map<String, Object> contentMap = messageDTO.getContentMap();
        if (MapUtils.isNotEmpty(contentMap)) {
            Set<String> keySet = contentMap.keySet();
            for (String k : keySet) {
                Object o = contentMap.get(k);
                text += k + "：" + o + "\n\r";
            }
        }
        if (StringUtils.isNotBlank(messageDTO.getTraceId())) {
            String traceUrl = "http://10.105.8.172:8780/damo-stats/mongodb/getLogMessagesView?trace_id=" + messageDTO.getTraceId();
            text += "链路：[" + messageDTO.getTraceId() + "](" + traceUrl + ")";
        }
        markdownMap.put("text", text);
        String post = HttpUtil.post(dingUrl, JSONObject.toJSONString(dingMap));
        logger.info("alert:{}", post);

    }
}
