package com.gic.haoban.manage.service.service.out.impl;

import java.util.Date;
import java.util.List;
import java.util.Set;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;

import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanWelcomeMapper;
import com.gic.haoban.manage.service.entity.TabMiniprogramSetting;
import com.gic.haoban.manage.service.entity.TabWelcome;
import com.gic.haoban.manage.service.service.MiniprogramSettingService;
import com.gic.haoban.manage.service.service.SecretSettingService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.service.dao.mapper.WxEnterpriseMapper;
import com.gic.haoban.manage.service.entity.TabHaobanWxApplication;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.service.WxApplicationService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;

@Service
public class WxEnterpriseApiServiceImpl implements WxEnterpriseApiService {
	private static final Logger log = LogManager.getLogger(WxEnterpriseApiServiceImpl.class);
	@Autowired
	private WxEnterpriseService wxEnterpriseService;
	@Autowired
	private StaffApiService staffApiService;
	@Autowired
	private WxEnterpriseMapper wxEnterpriseMapper;
	@Autowired
	private WxApplicationService wxApplicationService;
	
	@Autowired
	private EnterpriseService enterpriseService;

	@Autowired
	private SecretSettingService secretSettingService;

	@Autowired
	private MiniprogramSettingService miniprogramSettingService;

	@Autowired
	private TabHaobanWelcomeMapper welcomeMapper;
	
	@Override
	public void callbackReviceEnterprise(QywxCorpInfoSimpleDTO dto) {
		log.info("企业微信授权信息：{}", JSON.toJSONString(dto));
		WxEnterpriseDTO wxDTO = EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, dto);
		TabHaobanWxEnterprise enterprise = this.wxEnterpriseService.getEnterpriseBycorpIdNoStatus(dto.getCorpid());
		String wxEnterpriseId = "";
		if(enterprise != null){
			wxEnterpriseId = enterprise.getWxEnterpriseId();
			wxDTO.setWxEnterpriseId(enterprise.getWxEnterpriseId());
			wxDTO.setStatusFlag(1);
            wxDTO.setBindFlag(1);
            wxEnterpriseService.update(wxDTO);
		}else{
			wxEnterpriseId = wxEnterpriseService.add(wxDTO);
		}
		String userId = dto.getUserid();
		String name = dto.getName();
		if(StringUtils.isNotBlank(userId)) {
			staffApiService.wxGetAdd(userId,wxEnterpriseId);
		}
		String siteId = dto.getSuiteId();
		String agentId = dto.getAgentid();
		String agentName = dto.getAgentName();
		String permanentCode = dto.getPermanentCode();
		String applicationName = dto.getName();
		
		TabHaobanWxApplication tab = wxApplicationService.selectBySiteIdAndWxEnterpriseId(siteId,wxEnterpriseId);
		if(tab == null) {
			tab = new TabHaobanWxApplication();
			tab.setAgentId(agentId);
			tab.setSiteId(siteId);
			tab.setAgentName(agentName);
			tab.setName(applicationName);
			tab.setPermanentCode(permanentCode);
			tab.setWxEnterpriseId(wxEnterpriseId);
			wxApplicationService.insert(tab);
		}else {
			tab.setPermanentCode(permanentCode);
			wxApplicationService.update(tab);
		}
		log.info("企业微信授权处理完成");
	}

	@Override
	public com.gic.api.base.commons.Page<YwWxEnterpriseDTO> list(BasePageInfo pageInfo,String keyword,List<String> enterpriseIds,Integer gicContactFlag,String version) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		Page<TabHaobanWxEnterprise>  page = wxEnterpriseMapper.list(keyword,enterpriseIds,gicContactFlag,version);
		return  PageUtil.changePageHelperToCurrentPage(page, YwWxEnterpriseDTO.class);
	}
	
	@Override
	public List<YwWxEnterpriseDTO> listAll() {
		List<TabHaobanWxEnterprise>  list = wxEnterpriseMapper.listAll();
		return  EntityUtil.changeEntityListByJSON(YwWxEnterpriseDTO.class, list);
	}
	
	@Override
	public List<YwWxEnterpriseDTO> listByIds(Set<String> enterpriseIds) {
		List<TabHaobanWxEnterprise>  list = wxEnterpriseMapper.listByIds(enterpriseIds);
		return  EntityUtil.changeEntityListByJSON(YwWxEnterpriseDTO.class, list);
	}

	@Override
	public WxEnterpriseDTO getOne(String wxEnterpriseId) {
		TabHaobanWxEnterprise tab = wxEnterpriseMapper.selectByPrimaryKey(wxEnterpriseId);
		WxEnterpriseDTO wxDTO = EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, tab);
		return wxDTO;
	}

	@Override
	public WxEnterpriseDTO getEnterpriseBycorpId(String corpId) {
		TabHaobanWxEnterprise enterprise = this.wxEnterpriseService.getEnterpriseBycorpId(corpId);
		return EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, enterprise);
	}

	@Override
	public void update(WxEnterpriseDTO enterpriseDTO) {
		wxEnterpriseService.update(enterpriseDTO);
		
	}
	
	@Override
	public boolean enterpriseIsOver(String enterpriseId) {
		if(StringUtils.isEmpty(enterpriseId)){
			return true;
		}
		EnterpriseDTO enterprise = enterpriseService.getEnterpriseByEid(enterpriseId);
		if(enterprise == null || enterprise.getExpireTime() == null){
			return true;
		}
		if(enterprise.getExpireTime().getTime() > System.currentTimeMillis()){
			return false;
		}
		return true;
		
	}

	@Override
	public SecretSettingDTO getSecretSetting(String wxEnterpriseId, int secretType, String enterpriseId) {
		if (secretType == SecretTypeEnum.MEMBER_WAPP.getVal()) {
			return secretSettingService.getMemberSecretSetting(wxEnterpriseId,enterpriseId);
		}else{
			return secretSettingService.getSecretSetting(wxEnterpriseId,secretType);
		}
	}

	@Override
	public boolean saveSecretSetting(SecretSettingDTO secretSettingDTO) {
		return secretSettingService.saveSecretSetting(secretSettingDTO);
	}

	@Override
	public List<SecretSettingDTO> listSecret(String wxEnterpriseId) {
		return secretSettingService.listSecretSetting(wxEnterpriseId);
	}

	@Override
	public boolean delSecretSetting(String secretId) {
		return secretSettingService.delSecretSetting(secretId);
	}

	@Override
	public boolean saveMiniprogramSetting(MiniprogramSettingDTO miniprogramSettingDTO) {
		miniprogramSettingService.saveMiniprogramSetting(EntityUtil.changeEntityNew(TabMiniprogramSetting.class, miniprogramSettingDTO));
		return true;
	}

	@Override
	public List<MiniprogramSettingDTO> listMiniprogramSetting(String wxEnterpriseId) {
		List<TabMiniprogramSetting> ret = miniprogramSettingService.listMiniprogram(wxEnterpriseId);
		return EntityUtil.changeEntityListNew(MiniprogramSettingDTO.class, ret);
	}

	@Override
	public MiniprogramSettingDTO getMiniprogramSettingByEid(String wxEnterpriseId, String enterpriseId) {
		TabMiniprogramSetting miniprogramSetting = miniprogramSettingService.getMiniprogramSetting(wxEnterpriseId, enterpriseId);
		return EntityUtil.changeEntityNew(MiniprogramSettingDTO.class, miniprogramSetting);
	}

	@Override
	public MiniprogramSettingDTO getMiniprogramSettingByAppId(String wxEnterpriseId, String appId) {
		TabMiniprogramSetting miniprogramSetting = miniprogramSettingService.getMiniprogramSettingByAppId(wxEnterpriseId, appId);
		return EntityUtil.changeEntityNew(MiniprogramSettingDTO.class, miniprogramSetting);
	}

	@Override
	public boolean delMiniprogramSettingByEid(String id) {
		miniprogramSettingService.delMiniprogramSetting(id);
		return true;
	}


	@Override
	public boolean saveWelcome(WelcomeDTO dto) {
		TabWelcome tabWelcome = welcomeMapper.selectByWxEnterpriseId(dto.getWxEnterpriseId());
		if (tabWelcome == null) {
            tabWelcome = new TabWelcome();
            tabWelcome.setTitle(dto.getTitle());
			tabWelcome.setWelcomeContent(dto.getWelcomeContent());
			tabWelcome.setCreateTime(new Date());
			tabWelcome.setUpdateTime(new Date());
			tabWelcome.setStatusFlag(1);
			tabWelcome.setOpenFlag(dto.getOpenFlag());
			tabWelcome.setWxEnterpriseId(dto.getWxEnterpriseId());
            tabWelcome.setWelcomeId(ToolUtil.randomUUID());
			welcomeMapper.insert(tabWelcome);
		} else {
			tabWelcome.setTitle(dto.getTitle());
			tabWelcome.setWelcomeContent(dto.getWelcomeContent());
			tabWelcome.setUpdateTime(new Date());
			tabWelcome.setOpenFlag(dto.getOpenFlag());
			welcomeMapper.updateByPrimaryKeySelective(tabWelcome);
		}
		return true;
	}

	@Override
	public WelcomeDTO getWelcome(String wxEnterpriseId) {
		TabWelcome tabWelcome = welcomeMapper.selectByWxEnterpriseId(wxEnterpriseId);
		return EntityUtil.changeEntityByJSON(WelcomeDTO.class, tabWelcome);
	}

	@Override
	public boolean setWelcome(String wxEnterpriseId, Integer openFlag) {
		TabWelcome tabWelcome = welcomeMapper.selectByWxEnterpriseId(wxEnterpriseId);
		if (tabWelcome == null) {
			return true;
		}
		tabWelcome.setOpenFlag(openFlag);
		tabWelcome.setUpdateTime(new Date());
		welcomeMapper.updateByPrimaryKey(tabWelcome);
		return true;
	}

	@Override
	public boolean delWelcome(String wxEnterpriseId) {
		TabWelcome tabWelcome = welcomeMapper.selectByWxEnterpriseId(wxEnterpriseId);
		if (tabWelcome == null) {
			return true;
		}
		tabWelcome.setStatusFlag(0);
		tabWelcome.setUpdateTime(new Date());
		welcomeMapper.updateByPrimaryKey(tabWelcome);
		return true;
	}
}
